# CHECK_BOOST_DEVEL
# --------------------
# Check whether the Boost headers are available
AC_DEFUN([CHECK_BOOST_DEVEL],
[AC_MSG_CHECKING([for Boost development files])
 AC_TRY_COMPILE(
    [@%:@include <boost/version.hpp>
     @%:@include <boost/shared_ptr.hpp>
     @%:@include <boost/assert.hpp>
     @%:@include <boost/current_function.hpp>],
    [],
    [AC_MSG_RESULT([yes])],
    [AC_MSG_RESULT([no])
     AC_MSG_ERROR([Boost development files not found])
    ])
])

# CHECK_BOOST_VERSION
# ----------------------
# Check whether the Boost installation is up to date
AC_DEFUN([CHECK_BOOST_VERSION],
[AC_MSG_CHECKING([Boost version])
 AC_REQUIRE([CHECK_BOOST_DEVEL])
 AC_TRY_COMPILE(
    [@%:@include <boost/version.hpp>],
    [@%:@if BOOST_VERSION < 103100
     @%:@error too old
     @%:@endif],
    [AC_MSG_RESULT([yes])],
    [AC_MSG_RESULT([no])
     AC_MSG_ERROR([outdated Boost installation])
    ])
])

# CHECK_BOOST_PROGRAM_OPTIONS
# ------------------------
# Check whether the Boost program options is available
AC_DEFUN([CHECK_BOOST_PROGRAM_OPTIONS],
[AC_MSG_CHECKING([for Boost program options])
 AC_REQUIRE([AC_PROG_CC])
 original_LIBS=$LIBS
 original_CXXFLAGS=$CXXFLAGS
 CC_BASENAME=`basename $CC`
 CC_VERSION=`echo "__GNUC__ __GNUC_MINOR__" | $CC -E -x c - | \
    tail -n 1 | $SED -e "s/ //"`
 for boost_lib in \
        boost_program_options-$CC_BASENAME$CC_VERSION \
        boost_program_options-$CC_BASENAME \
        boost_program_options \
        boost_program_options-mt-$CC_BASENAME$CC_VERSION \
        boost_program_options-$CC_BASENAME$CC_VERSION-mt \
        boost_program_options-x$CC_BASENAME$CC_VERSION-mt \
        boost_program_options-mt-$CC_BASENAME \
        boost_program_options-$CC_BASENAME-mt \
        boost_program_options-mt ; do
     LIBS="$original_LIBS -l$boost_lib"
     CXXFLAGS="$original_CXXFLAGS"
     boost_po_found=no
     AC_LINK_IFELSE(
         [@%:@include <boost/program_options.hpp>
          using namespace boost::program_options;
          int main(int argc, char** argv)
          {
              options_description desc("Allowed options");
              desc.add_options()
                  ("help", "help msg")("p","p"); 
              return 0;
          }
         ],
         [boost_po_found=$boost_lib
          break],
         [])
 done
 LIBS="$original_LIBS"
 CXXFLAGS="$original_CXXFLAGS"
 if test "$boost_po_found" = no ; then
     AC_MSG_RESULT([no])
     AC_SUBST([BOOST_PROGRAM_OPTIONS_LIB],[""])
     AC_MSG_WARN([Boost program options not found.])
 else
     AC_MSG_RESULT([yes])
     AC_SUBST([BOOST_PROGRAM_OPTIONS_LIB],[$boost_lib])
 fi
])

# CHECK_BOOST
# ------------------------
# Boost-related tests
AC_DEFUN([CHECK_BOOST],
[AC_REQUIRE([CHECK_BOOST_DEVEL])
 AC_REQUIRE([CHECK_BOOST_VERSION])
 AC_REQUIRE([CHECK_BOOST_PROGRAM_OPTIONS])
])

