// cl/algorithms/algorithm.hpp
#ifndef CLUSLIB_ALGORITHM_HPP
#define CLUSLIB_ALGORITHM_HPP

#include<cl/datasets/dataset.hpp>
#include<cl/distances/distance.hpp>
#include<map>
#include<boost/any.hpp>

namespace ClusLib {

    class Additional {
    public:
        const boost::any& get(const std::string &name) const;
        void insert(const std::string &name, 
                    const boost::any &val);

        std::map<std::string, boost::any> additional;

    protected:
        Additional() {}
    };

    class Arguments : public Additional {
    public:
        boost::shared_ptr<Dataset> ds;
        boost::shared_ptr<Distance> distance; 
    };

    class Results : public Additional {
    public:
        void reset(); 

        std::vector<Size> CM;
    }; 

    class Algorithm {
    public:
        virtual ~Algorithm() {}
        virtual Arguments& getArguments(); 
        virtual const Results& getResults() const; 
        virtual void reset() const; 
        virtual void clusterize();

    protected:
        virtual void setupArguments();
        virtual void performClustering() const = 0;
        virtual void fetchResults() const  = 0;

        boost::shared_ptr<Dataset> _ds;
        mutable Results _results;
        Arguments _arguments;
    };
}

#endif
