// cl/algorithms/cmean.hpp
#ifndef CLUSLIB_CMEAN_HPP
#define CLUSLIB_CMEAN_HPP

#include<cl/algorithms/algorithm.hpp>
#include<cl/algorithms/kmean.hpp>
#include<cl/types.hpp>
#include<cl/datasets/dataset.hpp>
#include<cl/clusters/centercluster.hpp>
#include<cl/clusters/pclustering.hpp>
#include<cl/distances/distance.hpp>
#include<cl/utilities/matrix.hpp>

namespace ClusLib {

    class Cmean: public Algorithm {
    private: 
        void setupArguments();
        void performClustering() const;
        void fetchResults() const;
        void initialization() const;
        void iteration() const;
        void updateCenter() const;
        void updateFCM() const;
        void calculateObj() const;

        mutable std::vector<boost::shared_ptr<CenterCluster> > 
            _clusters;
        mutable std::vector<Size> _CM;
        mutable ublas::matrix<Real> _FCM;
        mutable Size _numiter;
        mutable Real _dObj;
        
        Real _threshold;
        Real _alpha;
        Real _epsilon;
        Size _numclust;
        Size _maxiter;
        Size _seed;
        boost::shared_ptr<Distance> _distance;
    };

} 
#endif
