// cl/algorithms/diana.hpp
#ifndef CLUSLIB_DIANA_HPP
#define CLUSLIB_DIANA_HPP

#include<cl/algorithms/algorithm.hpp>
#include<cl/clusters/hclustering.hpp>
#include<cl/utilities/nnmap.hpp>
#include<cl/datasets/dataset.hpp>
#include<cl/distances/distance.hpp>
#include<cl/patterns/nodevisitor.hpp>
#include<cl/types.hpp>
#include<set>

namespace ClusLib {

    class Diana: public Algorithm { 
    protected:
        void setupArguments();
        void performClustering() const;
        void fetchResults() const;
        virtual void create_dm() const;
        virtual void initialization() const;
        virtual void division() const;
        virtual void do_split(Size ind) const;
        virtual void create_cluster(const std::set<Size> ele, 
            Size ind) const;

        mutable iirMapA _dm;
        mutable std::set<Size> _unsplitClusters;
        mutable std::map<Size,Real> _clusterDiameter;
        mutable std::map<Size,boost::shared_ptr<LeafNode> > _leaf;
        mutable std::map<Size,boost::shared_ptr<InternalNode> > 
            _internal;
        mutable std::set<Size> _clusterID;
        boost::shared_ptr<Distance> _distance;
    };
}

#endif
