// cl/algorithms/gkmode.hpp
#ifndef CLUSLIB_GKMODE_HPP
#define CLUSLIB_GKMODE_HPP

#include<cl/algorithms/algorithm.hpp>
#include<cl/clusters/centercluster.hpp>
#include<cl/clusters/pclustering.hpp>
#include<cl/datasets/dataset.hpp>
#include<cl/distances/distance.hpp>
#include<cl/utilities/matrix.hpp>

namespace ClusLib {

    class GKmode : public Algorithm {
    private:
        void setupArguments();
        void performClustering() const;
        void fetchResults() const;
        void initialization() const;
        void iteration() const;
        void selection(Size g) const;
        void mutation(Size g) const;
        void kmode(Size ind) const;
        void calculateLoss(Size ind) const;
        bool get_mode(Size &mode, Real &var, 
                Size ind, Size k, Size j) const; 
        Real calculateRatio(Size ind) const;
        void generateRN(std::vector<Size>& nv, Size s) const;


        mutable std::vector<boost::shared_ptr<CenterCluster> > 
            _clusters;
        mutable ublas::matrix<Size> _mP; 
        mutable std::vector<Real> _dvFit;
        mutable std::vector<bool> _bvLegal;

        Size _numclust;
        Size _numpop;
        Size _maxgen;
        Real _c;
        Real _cm;
        Real _Pm;
    };

}

#endif
