// cl/algorithms/gmc.hpp
#ifndef CLUSLIB_GMC_HPP
#define CLUSLIB_GMC_HPP

#include<cl/algorithms/algorithm.hpp>
#include<cl/clusters/centercluster.hpp>
#include<cl/datasets/dataset.hpp>
#include<cl/distances/distance.hpp>
#include<cl/utilities/matrix.hpp>

namespace ClusLib {

    class GMC: public Algorithm {
    protected: 
        void setupArguments();
        void performClustering() const;
        void fetchResults() const;
        virtual void initialization() const;
        virtual void iteration() const;
        virtual void estep() const;
        virtual void mstep() const;

        mutable ublas::matrix<Real> _mu;
        mutable std::vector<ublas::symmetric_matrix<Real> > 
            _sigma;
        mutable std::vector<Real> _p;
        mutable ublas::matrix<Real> _data;
        mutable Real _ll;
        mutable ublas::matrix<Real> _post;
        mutable std::vector<boost::shared_ptr<CenterCluster> > 
            _clusters;
        mutable std::vector<Size> _CM;
        mutable bool _converged;
        mutable Size _numiter;

        Size _seed;
        Size _numclust; 
        Size _maxiter;
        Real _threshold;
        Real _epsilon;
    };
}

#endif
