// cl/algorithms/kprototype.cpp
#include<cl/algorithms/kprototype.hpp>
#include<cl/errors.hpp>
#include<iostream> 
#include<map> 

namespace ClusLib {

    void Kprototype::setupArguments() { 
        Algorithm::setupArguments();

        _distance = _arguments.distance;
        ASSERT(_distance, "distance is null");

        _numclust = boost::any_cast<Size>(
            _arguments.get("numclust")); 
        ASSERT(_numclust>=2 && _numclust<=_ds->size(), 
            "invalid numclust");

        _maxiter = boost::any_cast<Size>(
            _arguments.get("maxiter")); 
        ASSERT(_maxiter>0, "invalide maxiter");

        _seed = boost::any_cast<Size>(
            _arguments.get("seed")); 
        ASSERT(_seed>0, "invalide seed");
    }

    void Kprototype::updateCenter() const {
        Real dTemp;
        boost::shared_ptr<Schema> schema = _ds->schema();
        for(Size k=0;k<_clusters.size();++k){ 
            for(Size j=0;j<schema->size();++j){
                if((*schema)[j]->can_cast_to_c()) {
                    dTemp = 0.0;
                    for(Size i=0; i<_clusters[k]->size();++i){
                        boost::shared_ptr<Record> rec = 
                            (*_clusters[k])[i];
                        dTemp+=(*schema)[j]->get_c_val((*rec)[j]);
                    } 
                    (*schema)[j]->set_c_val(
                        (*_clusters[k]->center())[j],
                        dTemp/_clusters[k]->size());
                } else {
                    DAttrInfo da = (*schema)[j]->cast_to_d();
                    std::map<Size, Size> freq;
                    for(Size i=0;i<da.num_values(); ++i){
                        freq.insert(
                            std::pair<Size,Size>(i, 0));
                    }

                    for(Size i=0; i<_clusters[k]->size();++i){
                        boost::shared_ptr<Record> rec = 
                            (*_clusters[k])[i];
                        freq[(*schema)[j]->get_d_val((*rec)[j])] 
                            += 1;
                    } 

                    Size nMax = 0;
                    Size s = 0;
                    for(Size i=0;i<da.num_values(); ++i){
                        if(nMax < freq[i]) {
                            nMax = freq[i];
                            s = i;
                        }
                    }
                    da.set_d_val((*_clusters[k]->center())[j], s); 
                }
            } 
        }
    } 
}
