// cl/algorithms/lw.cpp
#include<cl/algorithms/lw.hpp>
#include<cl/types.hpp>
#include<iostream>

namespace ClusLib {

    void LW::setupArguments() {
        Algorithm::setupArguments();

        _distance = _arguments.distance;
        ASSERT(_distance, "distance is null");
    }

    void LW::create_dm() const { 
        Size n = _ds->size();
        for(Size i=0;i<n-1;++i){
            for(Size j=i+1;j<n;++j){
                _dm.add_item(i, j, 
                      (*_distance)((*_ds)[i],(*_ds)[j]));
            }
        }
    }

    void LW::performClustering() const {
        create_dm();
        init_forest();
        linkage();
    }

    void LW::init_forest() const {
        Size n = _ds->size(); 
        for(Size s=0;s<n;++s){
            boost::shared_ptr<Node> pln(new 
                LeafNode((*_ds)[s], s));
            pln->set_level(0);
            boost::shared_ptr<HClustering> phc(new 
                HClustering(pln));
            _forest.insert(Forest::value_type(s, phc));
            _clusterSize.insert(SizeMap::value_type(s,1));
            _unmergedClusters.insert(s);
        }
    }

    void LW::linkage() const {
        Size n = _ds->size();
        std::set<Size>::iterator it;
        Real dMin, dTemp;
        Size m, s1, s2;
        for(Size s=0;s<n-1;++s){
            dMin = MAX_REAL;
            std::vector<Integer> nvTemp(_unmergedClusters.begin(),
                    _unmergedClusters.end());
            m = nvTemp.size();
            for(Size i=0;i<m;++i) {
                for(Size j=i+1;j<m;++j){
                    dTemp = _dm(nvTemp[i],nvTemp[j]);
                    if(dTemp < dMin) {
                        dMin = dTemp;
                        s1 = nvTemp[i];
                        s2 = nvTemp[j];
                    }
                }
            }
            boost::shared_ptr<Node> node = 
                _forest[s1]->joinWith(*_forest[s2],dMin);
            node->set_id(n+s);
            node->set_level(s+1);
            boost::shared_ptr<HClustering> phc =
                boost::shared_ptr<HClustering>(new 
                HClustering(node)); 
            _forest.insert(Forest::value_type(n+s, phc));
            _clusterSize.insert(SizeMap::value_type(n+s,
                _clusterSize[s1]+_clusterSize[s2]));
            _unmergedClusters.erase(s1);
            _unmergedClusters.erase(s2);
            _unmergedClusters.insert(n+s);
            update_dm(s1, s2, n+s); 
        }
    }

    void LW::fetchResults() const {
        Size n = _ds->size();
        _results.insert("hc",HClustering(_forest[2*n-2]->root()));
    }
}
