// cl/algorithms/ward.cpp
#include<cl/algorithms/ward.hpp>
#include<cl/distances/euclideandistance.hpp>
#include<cmath>

namespace ClusLib {

    void Ward::setupArguments() {
        Algorithm::setupArguments();

        _distance = boost::shared_ptr<Distance>(new
            EuclideanDistance());
    }

    void Ward::update_dm(Size p, Size q, Size r) const {
        Real dist;
        std::set<Size>::iterator it;
        Real sp = _clusterSize[p];
        Real sq = _clusterSize[q];
        for(it = _unmergedClusters.begin();
            it != _unmergedClusters.end(); ++it) {
            if(*it == r) {
                continue;
            }

            Real sk = _clusterSize[*it];
            Real st = sp+sq+sk;
            dist = std::pow(_dm(p,*it), 2.0)*(sp+sk)/st + 
                   std::pow(_dm(q,*it), 2.0)*(sk+sq)/st - 
                   std::pow(_dm(p,q), 2.0)*sk/st;
            _dm.add_item(r,*it, std::sqrt(dist));
        }
    }

}
