// cl/datasets/attrinfo.hpp
#ifndef CLUSLIB_ATTRINFO_HPP
#define CLUSLIB_ATTRINFO_HPP

#include<cl/datasets/attrvalue.hpp>
#include<string>

namespace ClusLib {

    enum AttrType {
        Unknow,
        Continuous,
        Discrete
    };

    class DAttrInfo;
    class CAttrInfo;

    class AttrInfo { 
    public:
        AttrInfo(const std::string &name, AttrType type);
        virtual ~AttrInfo() {}

        const std::string& name() const;
        std::string& name();
        AttrType type() const; 

        virtual bool operator==(const AttrInfo& info) const;
        virtual bool operator!=(const AttrInfo& info) const;
        virtual AttrInfo* clone() const = 0;
        virtual Real distance(const AttrValue&, 
			      const AttrValue&) const = 0;
        virtual void set_d_val(AttrValue&, Size) const;
        virtual Size get_d_val(const AttrValue&) const;
        virtual void set_c_val(AttrValue&, Real) const;
        virtual Real get_c_val(const AttrValue&) const;
        virtual void set_unknown(AttrValue&) const = 0;
        virtual bool is_unknown(const AttrValue&) const = 0;
        virtual DAttrInfo& cast_to_d();
        virtual const DAttrInfo& cast_to_d() const;
        virtual CAttrInfo& cast_to_c();
        virtual const CAttrInfo& cast_to_c() const;
        virtual bool can_cast_to_d() const;
        virtual bool can_cast_to_c() const;

    protected:
        bool equal_shallow(const AttrInfo&) const;

    private: 
        std::string _name;
        AttrType _type;
    };

    inline const std::string& AttrInfo::name() const {
        return _name;
    }

    inline std::string& AttrInfo::name() {
        return _name;
    }

    inline AttrType AttrInfo::type() const {
        return _type;
    }

    inline bool AttrInfo::operator==(const AttrInfo& info) const {
        return equal_shallow(info);
    }

    inline bool AttrInfo::operator!=(const AttrInfo& info) const {
        return !equal_shallow(info);
    }

    inline bool AttrInfo::can_cast_to_d() const {
        return false;
    }

    inline bool AttrInfo::can_cast_to_c() const {
        return false;
    }

}

#endif
