// cl/datasets/dataset.hpp
#ifndef CLUSLIB_DATASET_HPP
#define CLUSLIB_DATASET_HPP

#include<cl/datasets/record.hpp>
#include<cl/datasets/schema.hpp>
#include<cl/utilities/container.hpp>
#include<vector>
#include<iostream>

namespace ClusLib {

    class Dataset: public Container<boost::shared_ptr<Record> > {
    public: 
        friend std::ostream& operator<<(std::ostream& os,
                const Dataset& ds);

        Dataset(const boost::shared_ptr<Schema>&);
        Dataset(const Dataset&);

        Size num_attr() const;
        const boost::shared_ptr<Schema>& schema() const;
        AttrValue& operator()(Size i, Size j);
        const AttrValue& operator()(Size i, Size j) const;
        bool is_numeric() const;
        bool is_categorical() const;
        void save(const std::string& filename) const;
        std::vector<Size> get_CM() const; 
        Dataset& operator=(const Dataset&);

    protected: 
        void print(std::ostream& os) const;

        boost::shared_ptr<Schema> _schema;
    };

    inline Size Dataset::num_attr() const {
        return _schema->size();
    }

    inline const boost::shared_ptr<Schema>& Dataset::schema() 
        const {
        return _schema;
    }

    inline AttrValue& Dataset::operator()(Size i, Size j) {
        return (*_data[i])[j];
    }

    inline const AttrValue& 
        Dataset::operator()(Size i, Size j) const {
        return (*_data[i])[j];
    }

} 

#endif
