// cl/datasets/dattrinfo.hpp
#ifndef CLUSLIB_DATTRINFO_HPP
#define CLUSLIB_DATTRINFO_HPP

#include<cl/datasets/attrinfo.hpp>
#include<vector>

namespace ClusLib {

    class DAttrInfo : public AttrInfo {
    public:
        DAttrInfo(const std::string& name);

        Size num_values() const;
        const std::string& int_to_str(Size i) const;
        Size str_to_int(const std::string&) const;
        Size add_value(const std::string&, 
            bool bAllowDuplicate = true); 
        void remove_value(const std::string&);
        void remove_value(Size i);
        DAttrInfo* clone() const;
        Real distance(const AttrValue&, const AttrValue&) const;
        void set_d_val(AttrValue&, Size) const;
        Size get_d_val(const AttrValue&) const;
        void set_unknown(AttrValue&) const;
        bool is_unknown(const AttrValue&) const;
        DAttrInfo& cast_to_d();
        const DAttrInfo& cast_to_d() const;
        bool can_cast_to_d() const;
        bool operator==(const AttrInfo& info) const;
        bool operator!=(const AttrInfo& info) const;

    protected: 
        typedef std::vector<std::string>::iterator iterator;
        typedef std::vector<std::string>::const_iterator 
	    const_iterator;

        bool equal(const AttrInfo&) const;

        std::vector<std::string> _values;
    };

    inline Size DAttrInfo::get_d_val(
	const AttrValue& av) const {
        return boost::get<Size>(av._value);
    }

    inline DAttrInfo& DAttrInfo::cast_to_d() {
        return *this;
    }

    inline const DAttrInfo& 
	DAttrInfo::cast_to_d() const {
        return *this;
    }

    inline bool DAttrInfo::can_cast_to_d() const {
        return true;
    }

    inline bool DAttrInfo::operator==(const AttrInfo& info)
       	const {
        return equal(info);
    }

    inline bool DAttrInfo::operator!=(const AttrInfo& info)
       	const {
        return !equal(info);
    }

}

#endif
