// cl/datasets/record.hpp
#ifndef CLUSLIB_RECORD_HPP
#define CLUSLIB_RECORD_HPP

#include<boost/shared_ptr.hpp>
#include<cl/types.hpp>
#include<cl/errors.hpp>
#include<cl/datasets/schema.hpp>
#include<cl/datasets/attrinfo.hpp>
#include<cl/utilities/container.hpp>
#include<vector>

namespace ClusLib {

    class Record: public Container<AttrValue> {
    public: 
        Record(const boost::shared_ptr<Schema>& schema);

        const boost::shared_ptr<Schema>& schema() const;
        AttrValue& labelValue();
        const AttrValue& labelValue() const;
        AttrValue& idValue();
        const AttrValue& idValue() const;
        Size get_id() const;
        Size get_label() const;

    private: 
        boost::shared_ptr<Schema> _schema; 
        AttrValue _label;
        AttrValue _id;
    };

    inline const boost::shared_ptr<Schema>& Record::schema() 
    		const { 
        return _schema;
    }

    inline AttrValue& Record::labelValue() {
        return _label;
    }

    inline const AttrValue& Record::labelValue() const {
        return _label;
    }

    inline AttrValue& Record::idValue() {
        return _id;
    }

    inline const AttrValue& Record::idValue() const {
        return _id;
    }

    inline Size Record::get_id() const {
        return _schema->idInfo()->get_d_val(_id);
    }

    inline Size Record::get_label() const {
        return _schema->labelInfo()->get_d_val(_label);
    }

} 

#endif
