// cl/datasets/schema.hpp
#ifndef CLUSLIB_SCHEMA_HPP
#define CLUSLIB_SCHEMA_HPP

#include<cl/types.hpp>
#include<cl/datasets/attrinfo.hpp>
#include<cl/datasets/cattrinfo.hpp>
#include<cl/datasets/dattrinfo.hpp>
#include<cl/utilities/container.hpp>
#include<boost/shared_ptr.hpp>
#include<vector> 

namespace ClusLib {

    class Record;

    class Schema: public Container<boost::shared_ptr<AttrInfo> > {
    public:
        virtual ~Schema() {}

        Schema* clone() const;
        boost::shared_ptr<DAttrInfo>& labelInfo();
        const boost::shared_ptr<DAttrInfo>& labelInfo() const;
        boost::shared_ptr<DAttrInfo>& idInfo();
        const boost::shared_ptr<DAttrInfo>& idInfo() const;
        void set_label(boost::shared_ptr<Record>& r, 
                       const std::string& val);
        void set_id(boost::shared_ptr<Record>& r, 
                       const std::string& val);
        bool is_labelled() const;

        virtual bool equal(const Schema& o) const;
        virtual bool equal_no_label(const Schema& o) const;
        virtual bool operator==(const Schema& o) const;
        virtual bool operator!=(const Schema& o) const;
        virtual bool is_member(const AttrInfo& info) const;

    protected:
        boost::shared_ptr<DAttrInfo> _labelInfo;
        boost::shared_ptr<DAttrInfo> _idInfo;
    };

    inline bool Schema::operator==(const Schema& o) const {
        return equal(o);
    }

    inline bool Schema::operator!=(const Schema& o) const {
        return !equal(o);
    }

    inline boost::shared_ptr<DAttrInfo>& Schema::labelInfo() {
        return _labelInfo;
    }

    inline const boost::shared_ptr<DAttrInfo>& Schema::labelInfo() 
        const {
        return _labelInfo;
    } 

    inline boost::shared_ptr<DAttrInfo>& Schema::idInfo() {
        return _idInfo;
    }

    inline const boost::shared_ptr<DAttrInfo>& Schema::idInfo() 
        const {
        return _idInfo;
    } 
} 
#endif
