// cl/distances/mahalanobisdistance.cpp
#include<cl/distances/mahalanobisdistance.hpp>
#include<cl/datasets/record.hpp>
#include<cmath>

namespace ClusLib {

    Real MahalanobisDistance::operator()(
        const boost::shared_ptr<Record> &x, 
        const boost::shared_ptr<Record> &y) const {
        boost::shared_ptr<Schema> schema = x->schema();
        ASSERT(*schema==*(y->schema()), "schema does not match");
        ASSERT(schema->size() == _A.size1(), 
                "record and matrix dimensions do not match");

        ublas::vector<Real> v(schema->size());
        for(Size i=0; i<schema->size(); ++i) {
            v(i) = (*schema)[i]->distance((*x)[i],(*y)[i]);
        }

        ublas::vector<Real> w;
        w.resize(v.size());
        ublas::axpy_prod(_A, v, w, true);

        return std::sqrt(ublas::inner_prod(w,w));
    }

}
