// cl/distances/mahalanobisdistance.hpp
#ifndef CLUSLIB_MAHALANOBISDISTANCE_HPP
#define CLUSLIB_MAHALANOBISDISTANCE_HPP

#include<cl/distances/distance.hpp>
#include<cl/errors.hpp>
#include<cl/types.hpp>
#include<cl/utilities/matrix.hpp>

namespace ClusLib {

    class MahalanobisDistance: public Distance {
    public:
        MahalanobisDistance(const 
            ublas::symmetric_matrix<Real> &sigma);
        Real operator()(const boost::shared_ptr<Record>&, 
            const boost::shared_ptr<Record>&) const;

    protected:
        ublas::triangular_matrix<Real> _A;
    };

    inline MahalanobisDistance::MahalanobisDistance(const
            ublas::symmetric_matrix<Real> &sigma)
        : Distance("Mahalanobis distance") {
        ublas::triangular_matrix<Real> Lm;
        Lm.resize(sigma.size1(), sigma.size1());
        _A.resize(sigma.size1(), sigma.size1()); 
        Size k = chol(sigma, Lm);
        ASSERT(k==0, "invalide covariance matrix");
        k = triangular_matrix_inverse(Lm, _A);
        ASSERT(k==0, "inversing triangular matrix failed"); 
    }

}

#endif
