// cl/patterns/pcvisitor.cpp
#include<cl/patterns/pcvisitor.hpp>
#include<cl/patterns/joinvaluevisitor.hpp>
#include<cl/types.hpp>
#include<iostream>

namespace ClusLib {

    CVisitor::CVisitor() {
        _cluster = boost::shared_ptr<Cluster>(new Cluster());
    }

    void CVisitor::visit(LeafNode& node) {
        _cluster->add(node.get_data());
    }

    void CVisitor::visit(InternalNode& node) {
        Integer n = node.num_children();
        for(Integer i=0;i<n;++i){
            node[i]->accept(*this);
        }
    }	

    PCVisitor::PCVisitor(PClustering &pc, Size cutlevel)
        : _pc(pc), _cutlevel(cutlevel) {
    }

    void PCVisitor::visit(LeafNode& node) {
        boost::shared_ptr<Cluster> c = 
            boost::shared_ptr<Cluster>(new Cluster());
        c->add(node.get_data()); 
        _pc.add(c);
    }

    void PCVisitor::visit(InternalNode& node) {
        if(node.get_level() >= _cutlevel) { 
            for(Size i=0;i<node.num_children();++i){
                node[i]->accept(*this);
            }
        } else {
            CVisitor cv;
            node.accept(cv);
            _pc.add(cv.get_cluster());
        }
    }
}
