// cl/utilities/datasetnormalizer.cpp
#include<cl/utilities/datasetnormalizer.hpp>
#include<sstream>

namespace ClusLib {

    DatasetNormalizer::DatasetNormalizer(const 
        boost::shared_ptr<Dataset> &ds): _ods(ds) {
        ASSERT(_ods, "input dataset is null");
    }

    void DatasetNormalizer::fill(boost::shared_ptr<Dataset> &ds) {
        get_minmax();

        boost::shared_ptr<Schema> schema(_ods->schema()->clone()); 
        ds = boost::shared_ptr<Dataset>(new Dataset(schema));

        for(Size i=0; i<_ods->size(); ++i) {
            boost::shared_ptr<Record> rec(new Record(schema));
            for(Size h=0; h<schema->size(); ++h) {
                if(! (*schema)[h]->can_cast_to_c()) {
                    (*schema)[h]->set_d_val((*rec)[h], 
                        (*schema)[h]->get_d_val((*_ods)(i,h)));
                } else {
                    Real dTemp = (
                        (*schema)[h]->get_c_val((*_ods)(i,h)) - 
                        _dvMin[h]) / ( _dvMax[h] - _dvMin[h]);
                    (*schema)[h]->set_c_val((*rec)[h], dTemp);
                }
            }

            if(schema->is_labelled()) { 
                const boost::shared_ptr<DAttrInfo> &label = 
                    schema->labelInfo();
                label->set_d_val(rec->labelValue(), 
                    (*_ods)[i]->get_label());
            }

            const boost::shared_ptr<DAttrInfo> &id = 
                schema->idInfo();
            id->set_d_val(rec->idValue(), (*_ods)[i]->get_id());

            ds->add(rec);
        }
    }

    void DatasetNormalizer::get_minmax() {
        boost::shared_ptr<Schema> schema = _ods->schema();
        _dvMin.resize(schema->size());
        _dvMax.resize(schema->size());

        for(Size h=0; h<schema->size(); ++h) {
            if(! (*schema)[h]->can_cast_to_c()) {
                continue;
            }

            Real dMin = MAX_REAL;
            Real dMax = MIN_REAL;
            Real dTemp;
            for(Size i=0; i<_ods->size(); ++i) { 
                dTemp = (*schema)[h]->get_c_val((*_ods)(i,h));
                if (dMin > dTemp ) {
                    dMin = dTemp;
                }

                if (dMax < dTemp ) {
                    dMax = dTemp;
                }
            }

            if( dMax - dMin < EPSILON) {
                dMax = dMin + 1.0;
            }

            _dvMin[h] = dMin;
            _dvMax[h] = dMax;
        }
    }
}
