// cl/utilities/matrix.cpp
#include<cl/utilities/matrix.hpp>
#include<cl/errors.hpp>
#include<cl/types.hpp>
#include<cmath>

namespace ClusLib {

    Size chol(const ublas::symmetric_matrix<Real>& A, 
              ublas::triangular_matrix<Real>& L) {
        using namespace ublas; 
        ASSERT(A.size1() == A.size2(), "matrix A is not square");
        ASSERT(L.size1() == L.size2(), "matrix L is not square");
        ASSERT(A.size1() == L.size1(), 
            "matrix A and matrix L have different dimensions");

        const Size n = A.size1();
        for (Size k=0; k < n; k++) { 
            double qL_kk = A(k,k) - inner_prod( 
                project( row(L, k), range(0, k) ),
                project( row(L, k), range(0, k) ) );

            if (qL_kk <= 0) {
                return 1 + k;
            } 

            double L_kk = sqrt( qL_kk );
            L(k,k) = L_kk;
              
            matrix_column<triangular_matrix<Real> > clk(L, k);
            project( clk, range(k+1, n) ) = ( 
                project( column(A, k), range(k+1, n) ) - prod( 
                    project(L, range(k+1, n), range(0, k)), 
                    project(row(L, k), range(0, k) ) ) ) / L_kk;
        }

        return 0;      
    }

    Size triangular_matrix_inverse(
          const ublas::triangular_matrix<Real>& L, 
          ublas::triangular_matrix<Real>& iL) {
        using namespace ublas; 
        ASSERT(L.size1() == L.size2(), "matrix L is not square");
        ASSERT(iL.size1() == iL.size2(),
            "matrix iL is not square");
        ASSERT(L.size1() == iL.size1(), 
            "matrix L and matrix iL have different dimensions");

        const Size n = L.size1();
        for (Size k=0; k < n; k++) { 
            if (std::fabs(L(k,k)) < EPSILON) {
                return 1 + k;
            } 
        }

        for (Size k=0; k < n; k++) { 
            iL(k,k) = 1 / L(k,k);
              
            for(Size j=k+1; j<n; ++j){
                iL(j, k) = - inner_prod(
                    project(row(L, j), range(k, j)),
                    project(column(iL, k), range(k, j)) )
                    / L(j,j);
            }
        }

        return 0;      
    }

}
