// examples/agglomerative/agglomerative.cpp
#include<cl/cluslib.hpp>

#include<boost/timer.hpp>
#include<iostream>
#include<fstream>
#include<iomanip>
#include<boost/program_options.hpp>

using namespace ClusLib;
using namespace boost::program_options;
using namespace std;

int main(int ac, char* av[]){
    try{
        options_description desc("Allowed options");
        desc.add_options()
            ("help", "produce help message")
            ("method", value<string>()->default_value("single"), 
                "method (single, complete, gaverage, wgaverage, \
                 centroid, median, ward)")
            ("datafile", value<string>(), "the data file")
            ("p", value<Size>()->default_value(50), 
             "maximum number of nodes to show in dendrogram")
            ("maxclust", value<Size>()->default_value(3), 
             "maximum number of clusters");

        variables_map vm;        
        store(parse_command_line(ac, av, desc), vm);
        notify(vm);    

        if (vm.count("help") || ac==1) {
            cout << desc << "\n";
            return 1;
        }

        string datafile;
        if (vm.count("datafile")) {
            datafile = vm["datafile"].as<string>();
        } else {
            cout << "Please provide a data file\n";
            return 1;
        }

        string method;
        if (vm.count("method") ){
            method = vm["method"].as<string>();
        }

        Size maxclust;
        if (vm.count("maxclust")) {
            maxclust = vm["maxclust"].as<Size>();
        } 
        Size p;
        if (vm.count("p")) {
            p = vm["p"].as<Size>();
        } 

        DatasetReader reader(datafile);
        boost::shared_ptr<Dataset> ds;
        reader.fill(ds);
        
        std::cout<<*ds<<std::endl;

        boost::shared_ptr<Algorithm> ca;
        boost::shared_ptr<Distance> dist(new EuclideanDistance());
        if (method == "single" ) {
            ca = boost::shared_ptr<Algorithm>(new Single());
        } else if (method == "complete") {
            ca = boost::shared_ptr<Algorithm>(new Complete());
        } else if (method == "gaverage") {
            ca = boost::shared_ptr<Algorithm>(new Average());
        } else if (method == "wgaverage") {
            ca = boost::shared_ptr<Algorithm>(new Weighted());
        } else if (method == "centroid") {
            ca = boost::shared_ptr<Algorithm>(new Centroid());
        } else if (method == "median") {
            ca = boost::shared_ptr<Algorithm>(new Median());
        } else if (method == "ward") {
            ca = boost::shared_ptr<Algorithm>(new Ward());
        } else {
            FAIL("method " << method << " is not available ");
        } 
        
        Arguments &Arg = ca->getArguments();
        Arg.ds = ds;
        Arg.distance = dist;

        boost::timer t;
        t.restart();
        ca->clusterize();
        double seconds = t.elapsed();
        std::cout<<"completed in "<<seconds<<" seconds"
            <<std::endl;

        std::string prefix;
        size_t ind = datafile.find_last_of('.');
        if(ind != std::string::npos ) {
            prefix = datafile.substr(0,ind);
        } else {
            prefix = datafile;
        }
        prefix += "-" + method + "-";

        const Results& Res = ca->getResults();

        HClustering hc = 
            boost::any_cast<HClustering>(Res.get("hc"));
        hc.save(prefix + "dendrogram.eps",p);
        JoinValueVisitor jv;
        hc.root()->accept(jv);
        
        std::string jvfile = prefix + "joinValues.csv";
        std::ofstream of;
        of.open(jvfile.c_str()); 
        of<<jv;
        of.close();

        PClustering pc = hc.get_pc(maxclust);
        std::cout<<pc<<std::endl;
        pc.save(prefix + "pcsummary.txt"); 

        return 0;
    } catch (std::exception& e) {
        std::cout<<e.what()<<std::endl;
        return 1;
    } catch (...){
        std::cout<<"unknown error"<<std::endl;
        return 2;
    }
}
