// examples/datasetgenerator/datasetgenerator.cpp
#include<cl/utilities/datasetgenerator.hpp>
#include<iostream>
#include<sstream>
#include<iomanip>

using namespace ClusLib;
using namespace std;

int main(int argc, char* argv[]) { 
    try{
        stringstream ss;
        Size N=60;
        if(argc>1) { 
            ss<<argv[1];
        }
        ss>>N;
        ublas::matrix<Real> mu(3,2);
        vector<ublas::symmetric_matrix<Real> > sigma(3);
        vector<Size> records(3,N);
        mu(0,0) = 0.0; mu(0,1) = 0.0;
        mu(1,0) = 10.0; mu(1,1) = 0.0;
        mu(2,0) = 5.0; mu(2,1) = 5.0;
        for(Size i=0; i<3; ++i) {
            sigma[i].resize(2);
            sigma[i](0,0) = 1.0;
            sigma[i](1,0) = -0.5 + i*0.5;
            sigma[i](1,1) = 1.0;
        }
        DataAdapter *p = new DatasetGenerator(mu, sigma, records);
        boost::shared_ptr<Dataset> ds;
        p->fill(ds); 
        delete p;

        ss.clear();
        ss.str("");
        ss<<3*N<<"points.csv";
        cout<<ss.str()<<endl;
        ds->save(ss.str());
        cout<<*ds<<endl;
        cout<<"Data: \n";
        cout<<setw(10)<<left<<"RecordID";
        for(Size j=0; j<ds->num_attr(); ++j) {
            stringstream ss;
            ss<<"Attribute("<<j+1<<")";
            cout<<setw(14)<<left<<ss.str();
        }
        cout<<setw(10)<<left<<"Label"<<endl;
        for(Size i=0; i<ds->size(); ++i) { 
            cout<<setw(10)<<left<<(*ds)[i]->get_id();
            for(Size j=0; j<ds->num_attr(); ++j) {
                boost::shared_ptr<Schema> schema = ds->schema();
                if((*schema)[j]->can_cast_to_c()) {
                    cout<<setw(14)<<left
                        <<(*schema)[j]->get_c_val((*ds)(i,j));
                } else {
                    cout<<setw(14)<<left
                        <<(*schema)[j]->get_d_val((*ds)(i,j));
                }
            }
            cout<<setw(10)<<left<<(*ds)[i]->get_label()<<endl;
        }

        return 0;
    } catch (exception& e) {
        cout<<e.what()<<endl;
        return 1;
    } catch (...) {
        cout<<"unknown error"<<endl;
        return 2;
    }
}
