// examples/dummy/dummy.cpp
#include<cl/cluslib.hpp>

#include<iostream>
#include<cstdlib>

using namespace ClusLib;
using namespace boost;

class DummyAlgorithm: public Algorithm {
private:
    void setupArguments() {
        Algorithm::setupArguments(); 
        _numclust = any_cast<Size>(
            _arguments.get("numclust"));
        ASSERT(_numclust>0 && _numclust<5,
            "numclust must in range (0, 5)");
    }

    void performClustering() const {
        for(Size i=0; i<4; ++i){
            _CM.push_back(i);
        }
    }

    void fetchResults() const {
        _results.CM = _CM;
        _results.insert("msg",
            any(std::string("A dummy clustering algorithm")));
    }

    mutable std::vector<Size> _CM; 
    Size _numclust;
};


int main(int argc, char* argv[]){
    try{
        Size numclust = 3;
        if (argc>1) {
            numclust = atoi(argv[1]);
        }

        boost::shared_ptr<Schema> schema(new Schema());
        boost::shared_ptr<Dataset> ds(new Dataset(schema)); 
        
        DummyAlgorithm ca;
        Arguments &Arg = ca.getArguments();
        Arg.ds = ds;
        Arg.insert("numclust", any(numclust));

        ca.clusterize();

        const Results& Res = ca.getResults();

        std::string msg = 
            boost::any_cast<std::string>(Res.get("msg"));
        std::cout<<"Cluster Membership Vector: \n";
        std::cout<<"    ";
        for(Size i=0; i<Res.CM.size(); ++i) { 
            std::cout<<Res.CM[i]<<" ";
        }
        std::cout<<"\nMessage: \n";
        std::cout<<msg<<'\n';

        return 0;
    } catch (std::exception& e) {
        std::cout<<e.what()<<std::endl;
        return 1;
    } catch (...){
        std::cout<<"unknown error"<<std::endl;
        return 1;
    }
}
