// examples/gkmode/gkmode.cpp
#include<cl/cluslib.hpp>

#include<boost/timer.hpp>
#include<boost/program_options.hpp>
#include<iostream>
#include<fstream>
#include<iomanip>

using namespace ClusLib;
using namespace std;
using namespace boost::program_options;

int main(int ac, char* av[]){
    try{
        options_description desc("Allowed options");
        desc.add_options()
            ("help", "produce help message")
            ("datafile", value<string>(), "the data file")
            ("k", value<Size>()->default_value(3), 
             "number of clusters")
            ("numpop", value<Size>()->default_value(50), 
             "number of chromosomes in the population")
            ("maxgen", value<Size>()->default_value(100), 
             "maximum number of generations")
            ("c", value<Real>()->default_value(1.5), 
             "parameter c")
            ("cm", value<Real>()->default_value(1.5), 
             "parameter c_m")
            ("pm", value<Real>()->default_value(0.2,"0.2"), 
             "mutation probability");

        variables_map vm;        
        store(parse_command_line(ac, av, desc), vm);
        notify(vm);    

        if (vm.count("help") || ac==1) {
            cout << desc << "\n";
            return 1;
        }

        string datafile;
        if (vm.count("datafile")) {
            datafile = vm["datafile"].as<string>();
        } else {
            cout << "Please provide a data file\n";
            return 1;
        }

        Size numclust = vm["k"].as<Size>(); 
        Size numpop = vm["numpop"].as<Size>(); 
        Size maxgen = vm["maxgen"].as<Size>(); 
        Real c = vm["c"].as<Real>();
        Real cm = vm["cm"].as<Real>();
        Real pm = vm["pm"].as<Real>();

        DatasetReader reader(datafile);
        boost::shared_ptr<Dataset> ds; 
        reader.fill(ds);
        std::cout<<*ds<<std::endl;

        GKmode ca;
        Arguments &Arg = ca.getArguments();
        Arg.ds = ds;
        Arg.insert("numclust", numclust);
        Arg.insert("numpop", numpop);
        Arg.insert("maxgen", maxgen);
        Arg.insert("c", c);
        Arg.insert("cm", cm);
        Arg.insert("pm", pm);

        boost::timer t;
        t.restart();

        ca.clusterize();

        double seconds = t.elapsed();
        std::cout<<"completed in "<<seconds<<" seconds"
            <<std::endl;

        const Results& Res = ca.getResults();

        PClustering pc = 
            boost::any_cast<PClustering>(Res.get("pc"));
        std::cout<<pc<<std::endl;

        return 0;
    } catch (std::exception& e) {
        std::cout<<e.what()<<std::endl;
        return 1;
    } catch (...){
        std::cout<<"unknown error"<<std::endl;
        return 2;
    }
}
