// examples/mpikmean/mpikmean.hpp
#ifndef CLUSLIB_MPIKMEAN_HPP
#define CLUSLIB_MPIKMEAN_HPP

#include<cl/cluslib.hpp>
#include<boost/mpi.hpp>
#include<boost/serialization/vector.hpp>
#include<boost/timer.hpp>

namespace ClusLib {

    template<typename T>
    struct vplus {
        std::vector<T> operator()(const std::vector<T> &x,
                const std::vector<T> &y) {
            std::vector<T> result = x;
            for(size_t i=0; i<x.size(); ++i) {
                result[i] += y[i];
            }
            return result;
        }
    };

    class MPIKmean: public Algorithm {
    protected: 
        void setupArguments();
        void performClustering() const;
        void fetchResults() const;
        virtual void initialization() const;
        virtual void iteration() const;
        virtual Real dist(Size i, Size j) const;

        mutable std::vector<Real> _centers;
        mutable std::vector<Real> _data;
        mutable Size _numObj;
        mutable Size _numAttr;
        mutable std::vector<Size> _CM;

        mutable std::vector<boost::shared_ptr<CenterCluster> > 
            _clusters;
        mutable Real _error;
        mutable Size _numiter;
        
        Size _numclust;
        Size _maxiter;
        Size _seed;
        boost::mpi::communicator _world;
    };

}


#endif
