// test-suite/attrinfo.cpp
#include"attrinfo.hpp"

#include<cl/datasets/cattrinfo.hpp>
#include<cl/datasets/dattrinfo.hpp>
#include<sstream>

using namespace ClusLib;
using namespace boost::unit_test_framework;

void AttrInfoTest::testDAttrInfo() { 
    std::stringstream ss;

    DAttrInfo nai("Nominal A");
    nai.add_value("A");
    nai.add_value("B");
    nai.add_value("A"); 
    ss<<nai.name()<<" has "<<nai.num_values()
        <<" values"<<std::endl;

    ss<<" B: "<<nai.str_to_int("B")<<std::endl;
    ss<<" A: "<<nai.str_to_int("A")<<std::endl;
    ss<<" 0: "<<nai.int_to_str(0)<<std::endl;
    ss<<" 1: "<<nai.int_to_str(1)<<std::endl;

    DAttrInfo naib("Nominal B"); 
    if(nai==naib)
        ss<<nai.name()<<"="<<naib.name()<<std::endl;
    if(nai!=naib)
        ss<<nai.name()<<"!="<<naib.name()<<std::endl;
    if(naib.can_cast_to_d())
        ss<<naib.name()<<" can cast to discrete"<<std::endl;
    else
        ss<<naib.name()<<" can not cast to discrete"<<std::endl;

    AttrValue nava, navb; 
    nai.set_unknown(nava);
    BOOST_CHECK(nai.is_unknown(nava)); 

    nai.set_d_val(nava,0);
    nai.set_d_val(navb,1);
    ss<<"nava navb distance: "<<nai.distance(nava,navb)
        <<std::endl;
    ss<<"nava has value: "<<nai.get_d_val(nava)<<std::endl;
    ss<<"navb has value: "<<nai.get_d_val(navb)<<std::endl;

    BOOST_MESSAGE(ss.str());
}

void AttrInfoTest::testCAttrInfo() {
    std::stringstream ss;

    CAttrInfo rai("Real A");
    CAttrInfo raib("Real B");
    CAttrInfo raic("Real B");

    if(raic==raib)
        ss<<raic.name()<<"="<<raib.name()<<std::endl;
    if(rai!=raib)
        ss<<rai.name()<<"!="<<raib.name()<<std::endl;

    if(rai.can_cast_to_c())
        ss<<rai.name()<<" can cast to continuous"<<std::endl;
    else
        ss<<rai.name()<<" can not cast to continuous"<<std::endl;

    AttrValue rava, ravb; 
    rai.set_unknown(rava);
    BOOST_CHECK(rai.is_unknown(rava)); 

    rai.set_c_val(rava,2.0);
    rai.set_c_val(ravb,4.0);

    ss<<"rava ravb distance: "<<rai.distance(rava,ravb)
        <<std::endl;
    ss<<"rava has value: "<<rai.get_c_val(rava)<<std::endl;
    ss<<"ravb has value: "<<rai.get_c_val(ravb)<<std::endl;

    BOOST_MESSAGE(ss.str());
}

test_suite* AttrInfoTest::suite() { 
    test_suite* suite = BOOST_TEST_SUITE(
        "Testing Attribute Infomation");

    suite->add(BOOST_TEST_CASE(&AttrInfoTest::testDAttrInfo));
    suite->add(BOOST_TEST_CASE(&AttrInfoTest::testCAttrInfo));

    return suite;
}

