// test-suite/matrix.cpp
#include"matrix.hpp"

#include<cl/utilities/matrix.hpp>
#include<boost/numeric/ublas/io.hpp>
#include<sstream>
#include<fstream>

using namespace ClusLib;
using namespace boost::unit_test_framework;
using namespace boost::numeric::ublas;

void MatrixTest::testCholesky() {
    BOOST_MESSAGE("Testing Cholesky"); 

    symmetric_matrix<Real> A(3,3);
    A(0,0) = 3.1325; 
    A(1,0) = 0.9748; A(1,1) = 1.4862;
    A(2,0) = -0.7613; A(2,1) = -0.8402; A(2,2) = 0.7390;

    triangular_matrix<Real> L(3,3);
    Size k = chol(A,L); 

    BOOST_CHECK(k==0);

    std::cout<<"A: \n";
    for(Size i=0; i<A.size1(); ++i) {
        std::cout<<row(A, i)<<std::endl;
    }
    std::cout<<"L: \n";
    for(Size i=0; i<L.size1(); ++i) {
        std::cout<<row(L, i)<<std::endl;
    }
}

void MatrixTest::testTriangularInverse() {
    BOOST_MESSAGE("Testing Triangular Inverse"); 

    triangular_matrix<Real> L(4,4);
    L(0,0) = 0.8281; 
    L(1,0) =-0.0512; L(1,1) = 0.4328;
    L(2,0) = 1.5389; L(2,1) = -0.5794; L(2,2) = 0.6421;
    L(3,0) = 0.6235; L(3,1) = -0.2072; L(3,2) = 0.3365; L(3,3)=0.1900;

    triangular_matrix<Real> iL(4,4);
    Size k = triangular_matrix_inverse(L,iL); 

    BOOST_CHECK(k==0);

    std::cout<<"L: \n";
    for(Size i=0; i<L.size1(); ++i) {
        std::cout<<row(L, i)<<std::endl;
    }
    std::cout<<"iL: \n";
    for(Size i=0; i<iL.size1(); ++i) {
        std::cout<<row(iL, i)<<std::endl;
    }
}

test_suite* MatrixTest::suite() { 
    test_suite* suite = BOOST_TEST_SUITE("Testing Matrix");

    suite->add(BOOST_TEST_CASE(&MatrixTest::testCholesky));
    suite->add(BOOST_TEST_CASE(&MatrixTest::testTriangularInverse));

    return suite;
}
