// test-suite/schema.cpp
#include"schema.hpp"
#include<cl/datasets/attrinfo.hpp> 
#include<cl/datasets/schema.hpp> 

using namespace ClusLib;
using namespace boost::unit_test_framework;

void SchemaTest::testSchema() { 
    BOOST_MESSAGE("Testing schema"); 
    boost::shared_ptr<Schema> schema(new Schema());
    schema->add(boost::shared_ptr<AttrInfo>(new CAttrInfo("Real 1")));
    schema->add(boost::shared_ptr<AttrInfo>(new DAttrInfo("Nominal 1")));
    schema->add(boost::shared_ptr<AttrInfo>(new CAttrInfo("Real 2")));

    DAttrInfo* p = dynamic_cast<DAttrInfo*>((*schema)[1].get());

    BOOST_REQUIRE(p != 0);
    p->add_value("A");
    p->add_value("B");
    p->add_value("C");
    p->add_value("B");
    
    boost::shared_ptr<DAttrInfo> p2(p->clone());
    std::stringstream ss;
    ss<<"Value\n";
    for(Size i=0;i<p2->num_values();++i){
        ss<<p2->int_to_str(i)<<"\n";
    }

    BOOST_CHECK(!schema->is_labelled());
    schema->labelInfo() = p2;
    BOOST_CHECK(schema->is_labelled());

    boost::shared_ptr<Schema> schema2(schema->clone());

    if(schema->equal_no_label(*schema2)) {
        ss<<"schema and schema2 are equal (no label)"<<std::endl;
    }

    if(schema->equal(*schema2)) {
        ss<<"schema and schema2 are equal"<<std::endl;
    }

    BOOST_MESSAGE(ss.str()); 
}

test_suite* SchemaTest::suite() { 
    test_suite* suite = BOOST_TEST_SUITE("Testing Schema");

    suite->add(BOOST_TEST_CASE(&SchemaTest::testSchema));

    return suite;
}
